/******************************************************************************
 *     GBS-Control: Custom settings for GBS8200/8220 RGBtoVGA convertor       *
 ******************************************************************************
 see  https://github.com/mybook4/DigisparkSketches/tree/master/GBS_Control
      http://shmups.system11.org/viewtopic.php?f=6&t=52172&start=180

 Ported to ATtiny861A and WinAVR by Bruce Abbott (bhabbott.net.nz)

 Connections:-

 ATtiny861A  GBS8200
  (DIP-20)    (P5)
    pin 1      SDA
    pin 2      LED -> 1k -> GND
    pin 3      SCL
    pin 5      VCC (+3.3V)
    pin 6      GND
    pin 11     SW1 -> GND (open = 288p, closed = 240p)

 100nF capacitor from pin 4 to pin 5

 Fuses: L = 0xE2
        H = 0xDF
        E = 0x01

 changes:-
 2015-5-25 V0.0  - it works!
 2015-7-02 V0.1  - updated switch wiring diagram
 2015-7-20 V0.2  - changed input resolutions to 288p (PAL) and 240p (NTSC)

*/
#include <avr/pgmspace.h>
//#define F_CPU (800000UL)
#include <util/delay.h>

#include "I2CBitBangWriter.h"

// Workaround for http://gcc.gnu.org/bugzilla/show_bug.cgi?id=34734
#undef PROGMEM
#define PROGMEM __attribute__((section(".progmem.data")))

#include "StartArray.h"
#include "288p_50Hz_1280x1024.h"
#include "240p_60Hz_1280x1024.h"


#define GBS_I2C_ADDRESS 0x17 // 0x2E


bool writeBytes(uint8_t slaveAddress, uint8_t slaveRegister, uint8_t* values, uint8_t numValues)
{
  /*
   Write to Consecutive Control Registers:
    -Start Signal
    -Slave Address Byte (R/W Bit = Low)
    -Base Address Byte
    -Data Byte to Base Address
    -Data Byte to (Base Address + 1)
    -Data Byte to (Base Address + 2)
    -Data Byte to (Base Address + 3)
    ..................
    -Stop Signal
  */

  I2CBitBangWriter.setSlaveAddress(slaveAddress);

  I2CBitBangWriter.addByteForTransmission(slaveRegister);

  I2CBitBangWriter.addBytesForTransmission(values, numValues);

  if(!I2CBitBangWriter.transmitData())
  {
    return false;
  }

  return true;
}

bool writeBytes(uint8_t slaveRegister, uint8_t* values, int numValues)
{
  return writeBytes(GBS_I2C_ADDRESS, slaveRegister, values, numValues);
}

bool writeOneByte(uint8_t slaveRegister, uint8_t value)
{
  return writeBytes(slaveRegister, &value, 1);
}

bool writeStartArray()
{
  for(unsigned int z = 0; z < ((sizeof(startArray)/2) - 1520); z++)
  {
    writeOneByte(pgm_read_byte(startArray + (z*2)), pgm_read_byte(startArray + (z*2) + 1));
    _delay_us(10);
  }

  return true;
}



bool writeProgramArray(const uint8_t* programArray)
{
  for(int y = 0; y < 6; y++)
  {
    writeOneByte(0xF0, (uint8_t)y );
    _delay_us(10);

    for(int z = 0; z < 15; z++)
    {
      uint8_t bank[16];
      for(int w = 0; w < 16; w++)
      {
        bank[w] = pgm_read_byte(programArray + (y*256 + z*16 + w));
      }

      writeBytes(z*16, bank, 16);

      _delay_us(10);
    }

  }

  return true;
}

// Macros for a "Resolution" switch which allows for toggling between
// 288p (switch off) and 240p (switch on)
//
// NOTE: The switch schematic below has an RC debouncer connected to it for stability
//
// The presence of this switch is optional.  The 50ms delay in the loop could be enough to mask the bounce of your switch.
// WARNING: Depending on switch/button connection, you may need to enable internal pull up resistors for pins.  This would
//
//
//   VCC -----/\/\/\/----+-----| SPDT switch |-----GND
//               10k     |            
//                       |          
//                       |
//                      PA7 (pin 11)
//
//   Note: resistor required only if not using internal pullup
//
#define RESOLUTION_SWITCH_DDR  DDRA
#define RESOLUTION_SWITCH_PIN  PINA
#define RESOLUTION_SWITCH_PORT PORTA
#define RESOLUTION_SWITCH_BIT  7

// This variable holds the last known state of the Resolution switch
// false = 288p, true = 240p
bool lastKnownResolutionSwitchState = false;


// onboard LED
#define LED_DDR DDRB
#define LED_PORT PORTB
#define LED_BIT 1


void setup() {

// We setup and read the value of the Resolution switch
// to determine if we are initially going
// to write 288p or 240p settings to the scaler chip
  RESOLUTION_SWITCH_DDR &= ~(1<<RESOLUTION_SWITCH_BIT); // Set the resolution switch to an input
  RESOLUTION_SWITCH_PORT |= (1<<RESOLUTION_SWITCH_BIT); // Turn on internal pullup
  bool currentResolutionSwitchState = ((RESOLUTION_SWITCH_PIN & (1<<RESOLUTION_SWITCH_BIT)) != 0); // Read the value of the Resolution switch

// NOTE: If a resolution switch isn't wanted, simply comment out the three code lines above and uncomment one the following lines
// bool currentResolutionSwitchState = false; // 288p
// bool currentResolutionSwitchState = true;  // 240p


  // Set up LED to show switch state
  LED_DDR |= (1<<LED_BIT);

  I2CBitBangWriter.initialize();

  // Write the start array
  writeStartArray();

  if(currentResolutionSwitchState == false) {
    // Write the 288p register values to the scaler chip
    writeProgramArray(programArray288p);
    LED_PORT |= (1<<LED_BIT); // turn on LED
  } else {
    // Write the 240p register values to the scaler chip
    writeProgramArray(programArray240p);
    LED_PORT &= ~(1<<LED_BIT); // turn off LED
  }

  lastKnownResolutionSwitchState = currentResolutionSwitchState;
}


int main() {
  setup();
  while(1){
    _delay_us(50); // delay 50 milliseconds

    // Read the state of the Resolution switch
    bool currentResolutionSwitchState = ((RESOLUTION_SWITCH_PIN & (1<<RESOLUTION_SWITCH_BIT)) != 0);

    if(currentResolutionSwitchState != lastKnownResolutionSwitchState)
    {
      // The switch state has been flipped
      // Write the appropriate register values to the scaler chip

      if(currentResolutionSwitchState == false) {
        // Write the 288p register values to the scaler chip
        writeProgramArray(programArray288p);
        LED_PORT |= (1<<LED_BIT); // turn on LED
      } else {
        // Write the 240p register values to the scaler chip
        writeProgramArray(programArray240p);
        LED_PORT &= ~(1<<LED_BIT); // turn off LED
      }

      lastKnownResolutionSwitchState = currentResolutionSwitchState;
    }
  }

}
